/**
 * Ono Project
 *
 * File:         Digger.java
 * RCS:          $Id: OessTimeoutDigger.java,v 1.1 2008/11/16 03:30:07 mas939 Exp $
 * Description:  Digger class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jun 13, 2006 at 8:55:41 AM
 * Language:     Java
 * Package:      edu.northwestern.ono.dns
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.ono.dns;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TimeZone;
import java.util.Map.Entry;

import org.xbill.DNS.DClass;
import org.xbill.DNS.ExtendedFlags;
import org.xbill.DNS.Message;
import org.xbill.DNS.Name;
import org.xbill.DNS.Record;
import org.xbill.DNS.ReverseMap;
import org.xbill.DNS.Section;
import org.xbill.DNS.SimpleResolver;
import org.xbill.DNS.TSIG;
import org.xbill.DNS.TextParseException;
import org.xbill.DNS.Type;

import edu.northwestern.ono.MainGeneric;
import edu.northwestern.ono.OnoConfiguration;
import edu.northwestern.ono.dht.IDistributedDatabase;
import edu.northwestern.ono.experiment.TraceRouteRunner;
import edu.northwestern.ono.position.CDNClusterFinder;
import edu.northwestern.ono.position.OnoPeerManager;
import edu.northwestern.ono.position.OnoPeerManager.OnoPeer;
import edu.northwestern.ono.stats.DNSEvent;
import edu.northwestern.ono.stats.EdgeServerRatio;
import edu.northwestern.ono.stats.PingResult;
import edu.northwestern.ono.stats.Statistics;
import edu.northwestern.ono.stats.DNSEvent.EventType;
import edu.northwestern.ono.ui.OnoView;
import edu.northwestern.ono.util.HashMapCache;
import edu.northwestern.ono.util.HashSetCache;
import edu.northwestern.ono.util.ILoggerChannel;
import edu.northwestern.ono.util.Pair;
import edu.northwestern.ono.util.PingManager;
import edu.northwestern.ono.util.PluginInterface;
import edu.northwestern.ono.util.Util;
import edu.northwestern.ono.util.Util.PingResponse;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 * 
 *         The Digger class hunts for Akamai or other CDN entries and reports
 *         them appropriately.
 * 
 *         Note: If DIG_FOR_OTHERS is set to true, you will have to make sure
 *         that you reduce the TCP timeout and increase the number of user ports
 *         available on your OS. For help, see
 *         http://www.minq.se/products/pureload/tuning/connections.html.
 */
public class OessTimeoutDigger extends Thread {
	
	private int sleepInterval = 600;
	private ILoggerChannel log;
	private boolean isActive;
	private Statistics stats;
	private PluginInterface pi;

	private static final boolean DEBUG = false;
	
	public OessTimeoutDigger() {
		super("OessTimeoutDigger");
		
	}
	
	/**
	 * Initialization code.
	 * 
	 * @param sleep
	 * @param l
	 * @param dd
	 * @param pi
	 */
	public void initialize(int sleep, ILoggerChannel l,
			IDistributedDatabase dd, PluginInterface pi) {
		sleepInterval = sleep;
		log = l;
		stats = Statistics.getInstance();
		this.pi = pi;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	@Override
	public void run() {

		org.xbill.DNS.Name name = null;
		int type = org.xbill.DNS.Type.A;
		int dclass = DClass.IN;
		boolean again = true;

		Message query;
		Message response = null;
		Record rec;
		SimpleResolver res = null;

		long b4 = System.currentTimeMillis();
		while (again) {
		  try {
			name = Name.fromString("test.ana-aqualab.cs.northwestern.edu", Name.root);
			res = new SimpleResolver("192.168.1.254");
			rec = Record.newRecord(name, type, dclass);
			query = Message.newQuery(rec);
			response = res.send(query);
			again = false;
		  } catch (IOException e1) {
			// TODO Auto-generated catch block
			System.out.println("TIMEOUT: do it again");
			again = true;
			
		  }
		}
		
		long after = System.currentTimeMillis() - b4;
		System.out.println("MARIO: " + Long.toString(after));
		if (true)
			return;
		
	}

	
}
